/***************************************************************************
 *             GSkyRegionCircle.hpp - Circular sky region class            *
 * ----------------------------------------------------------------------- *
 *  copyright (C) 2009-2013 by Juergen Knoedlseder                         *
 * ----------------------------------------------------------------------- *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 *                                                                         *
 ***************************************************************************/
/**
 * @file GSkyRegionCircle.hpp
 * @brief Circular sky region class interface definition
 * @author Pierrick Martin
 */

#ifndef GSKYREGIONCIRCLE_HPP
#define GSKYREGIONCIRCLE_HPP

/* __ Includes ___________________________________________________________ */
#include <string>
#include "GSkyRegion.hpp"
#include "GSkyDir.hpp"
#include "GXmlElement.hpp"


/***********************************************************************//**
 * @class GSkyRegionCircle
 *
 * @brief Circular sky region
 *
 * This class implements a sky region delimited by a circle over the sky.
 * The sky region has three parameters: the Right Ascension and Declination 
 * of the centre, and the radius of the circle.
 *
 ***************************************************************************/
class GSkyRegionCircle : public GSkyRegion {

public:
    // Constructors and destructors
    GSkyRegionCircle(void);
	GSkyRegionCircle(const GSkyRegionCircle& region);
    explicit GSkyRegionCircle(const GSkyDir& dir, const double& rad);
    explicit GSkyRegionCircle(const double& ra, const double& dec, const double& rad);
    explicit GSkyRegionCircle(const GXmlElement& xml);    
    virtual ~GSkyRegionCircle(void);

    // Operators
    virtual GSkyRegionCircle& operator=(const GSkyRegionCircle& region);

    // Implemented pure virtual methods
    virtual void         clear(void);
    virtual GSkyRegion*  clone(void) const;
    virtual std::string  type(void) const;
    virtual void         read(const GXmlElement& xml);
    virtual void         write(GXmlElement& xml) const;
    virtual std::string  print(const GChatter& chatter = NORMAL) const;

    // Other methods
    double  ra(void) const;
    double  dec(void) const;
    void    ra(const double& ra);
    void    dec(const double& dec);
    GSkyDir dir(void) const;
    void    dir(const GSkyDir& dir);
    double  rad(void) const;
    void    rad(const double& rad);

protected:
    // Protected methods
    void init_members(void);
    void copy_members(const GSkyRegionCircle& region);
    void free_members(void);

    // Protected members
    double m_ra;          //!< Right Ascension of centre (deg)
    double m_dec;         //!< Declination of centre (deg)
    double m_rad;         //!< Circle radius (deg)
};


/***********************************************************************//**
 * @brief Return sky region type
 *
 * @return "SkyRegionCircle".
 *
 * Returns the type of the sky region.
 ***************************************************************************/
inline
std::string GSkyRegionCircle::type(void) const
{
    return "SkyRegionCircle";
}


/***********************************************************************//**
 * @brief Return Right Ascension of region centre
 *
 * @return Right Ascension of region centre (degrees).
 *
 * Returns the Right Ascension of the region centre in degrees.
 ***************************************************************************/
inline
double GSkyRegionCircle::ra(void) const
{
    return (m_ra.value());
}


/***********************************************************************//**
 * @brief Set Right Ascension of region centre
 *
 * @param[in] ra Right Ascension of region centre.
 *
 * Sets the Right Ascension of region centre.
 ***************************************************************************/
inline
void GSkyRegionCircle::ra(const double& ra)
{
    m_ra.value(ra);
    return;
}


/***********************************************************************//**
 * @brief Return Declination of region centre
 *
 * @return Declination of region centre (degrees).
 *
 * Returns the Declination of the region centre in degrees.
 ***************************************************************************/
inline
double GSkyRegionCircle::dec(void) const
{
    return (m_dec.value());
}


/***********************************************************************//**
 * @brief Set Declination of region centre
 *
 * @param[in] dec Declination of region centre.
 *
 * Sets the Declination of region centre.
 ***************************************************************************/
inline
void GSkyRegionCircle::dec(const double& dec)
{
    m_dec.value(dec);
    return;
}

#endif /* GSKYREGIONCIRCLE_HPP */